% This demo file runs the SD Toolbox on the various examples proposed
% in the data files:
%   * Spacectaft1.m to Spacectaft5.m, Spacectaft1u.m
%   * SpaceRoboticArm.m
%   * FOUR_CMGs.m
% and performs some analyzes.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%     File: Spacecraft1.m
%
%     A rigid platform with 2 cantilevered 
%     flexible appendages,                    
%
clear all,close all,
mod=main('Spacecraft1');
% 1 flexible mode removed (due to symmetry of flexible appendages)
mod.TotalCg            % the position of the total center of mass G.
mod.liste_IOs          % list of channels
%
Md=mod.DynamicModel;         % direct dynamic model.
% Translation of the direct dynamic model from the whole center of mass G
% to the main platform center of mass B:
Spacecraft1            % to get all the data in the working space.
MD_B=translate_dynamic_model(mod.TotalCg,MB.cg,Md);
%
% Inverse Dynamic Model:
Mi=mod.InverseTotalModel;
damp(Mi)  % The 5=6-1 flexible modes.
figure
sigma(Mi) % Frequency domaine response.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%     File: Spacecraft2.m
%
%     The same platform but the first appendage is
%     connected to the main body through a pivot joint
%     and tilted with a 10 degres angle,
%
clear all,close all,
mod=main('Spacecraft2');
mod.liste_IOs % Now the dynamic model is 7x7
Md=mod.DynamicModel;         % direct dynamic model.
Mi=mod.InverseTotalModel;    % inverse dynamic model
Mi71=Mi(7,1);  % Transfer between a force applied on the
%               platform along X axis and the pivot
%               relative acceleration.
ddint=tf(1,[1 0 0]); % double integrations
% impulse response of Mi71/s/s :
figure
impulse(Mi71*ddint,3) % response of the pivot angle to
%                       an impulse on the X thruster.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%     File: Spacecraft5.m
%
%     same as Spacecraft1.m but with additionnal 3 appendages
%     corresponding to angular momentums along to X, Y and Z
%     axes (to represent spinned RWA)
%
clear all,close all,
mod=main('Spacecraft5');
% 1 flexible mode removed (due to symetry of flexible appendages)
% + 2 double integrators removed.
%
Md=mod.DynamicModel;         % direct dynamic model.
damp(Md) % only 2 integrators to represent the 3 angular momentums.
Mi=mod.InverseTotalModel;
damp(Mi) % ==> in addition to flexible modes, on can see a very low
%frequency (around 0.1 rd/s) mode corresponding to the gyroscopic mode
figure
sigma(Mi) % ==> a very low fequency resonance.
%
% Angular model:
Mi=Mi(4:6,4:6);
% Integrations between angular acceleration and rates:
int=tf(1,[1 0]);
Mi_int=minreal(Mi*(int*eye(3)));
% ==> 2 states removed: extra inegrators to model gyroscopic complings
%     are removed, as wanted.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%    File: SpaceRoboticArm.m
%
%    a platform fitting a 6 d.o.f. rigid space robotic arm: 
%    the 6 links of the arm are described in the files Segment1.m,
%    Segment2.m, ..., Segment6.m.
%     
%
clear all,close all,
global config
mod=main('SpaceRoboticArm');
mod.liste_IOs  % The 12 channels (d.o.f. order).
mod.TotalCg    % Position of the whole center of mass.
mod.DynamicModel
% This model is static since all links are rigid.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%      File: FOUR_CMGs.m
%
%      A rigid platform with 4 CMGs,  
%
clear all,close all,
mod=main('FOUR_CMGs');
mod.liste_IOs         % Definition of the various channels
Md=mod.DynamicModel;  % direct dynamic model
% 
% Model restricted to 3 axes angular motion + CMGs channels:
Md=Md(4:10,4:10);
%
% One can check that the total angular momentum is null:
M3x3=minreal(Md(1:3,1:3))
%  ==> The 3x3 model is static: no more gyroscopic couplings between platform
%  and inertial frame.
%
Mi=inv(Md);  % Inverse dynamic model.
damp(Mi)
% ==> the pulsations of the 3 gyroscopic modes.
%
% Dynamic model for a new angular configuration:
global angles
angles=[10;20;-5;-25];
mod_bis=main('FOUR_CMGs');
damp(mod_bis.InverseTotalModel)
% ==> pulsations of gyroscopic modes have changed.
%
% First integrators (angular rates on outputs on nominal model Mi):
int=tf(1,[1 0]);
Gv=minreal((int*eye(7))*Mi);  
% 3 removed state instead of 6.
%
% Precession rate servo-loop:
rltool(0.06*Gv(4,4));
rltool(0.06*Gv(5,5));
rltool(0.06*Gv(6,6));
rltool(0.06*Gv(7,7));
% bf21=feedback(Gv,2,4,4);
% rltool(0.01*bf21(7,7))
bf2=feedback(Gv*diag([1 1 1 0.06 0.06 0.06 0.06]),eye(4),4:7,4:7);
bf2=minreal(bf2);  % 3 states removed: OK !!
damp(bf2) 
% ==> gyroscopic modes are correctly damped.
%
% Platform attitude servo-loop:
% The jacobian is defined as the DCgain of the transfer between the 4  
% precession rates reference inputs and the 3 platform angular rates:
Jacob=dcgain(bf2(1:3,[4:7]))
% Jacob=real(freqresp(bf2(1:3,4:7),sqrt(-1)*0.01)); % on cas of pb with
% minreal.
%
% Second integrators: platform angular positions and rates on outputs:
sint=ss(0,1,[1;0],[0;1]);
Gpv=minreal(append(sint,sint,sint,1,1,1,1)*bf2);
%
% CMGs guidance:
Gpv=Gpv*[eye(3),zeros(3);zeros(4,3) pinv(Jacob)];
%
% Attitude servo-loop bandwitdh:
w=3;
bf=feedback(Gpv*diag([1,1,1,w,w,w]),eye(3),[4 5 6],[1 3 5]);
damp(bf)
%  ==> all the modes are correctly damped.
figure
step(bf([1 3 5],[4 5 6]),2);
% No comments !!

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%     File: Spacecraft1u.m
%
%     The same platform than Spacecraft1.m but  the
%     main mechanical parameters m, I, L, omega are
%     subject to variations (typically + or - 10%
%     of relative variations)
%
clear all,close all,
mod=main('Spacecraft1u');
mod.TotalCg                % the position of the total center of mass.
Mi=mod.InverseTotalModel;  % inverse dynamic model
figure
sigma(Mi)  % sensitivity of the frequency domain
%            response to parametric variations.

